﻿using System;
using System.Windows.Forms;

namespace Calculator
{
	public partial class Form1 : Form
	{
		/// <summary>
		/// 現在の計算結果を保持します
		/// </summary>
		private double result;
		/// <summary>
		/// 計算ボタンが押されて新しい数字の入力が必要かどうかを示します
		/// </summary>
		private bool isNew;
		/// <summary>
		/// 押された計算ボタンの種類を保持します
		/// </summary>
		private Operator operation;

		public Form1()
		{
			InitializeComponent();
		}

		#region フォーム表示

		/// <summary>
		/// 画面表示時の初期化処理を行います
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_Load(object sender, EventArgs e)
		{
			this.result = double.MinValue;
			this.isNew = true;
			this.operation = Operator.None;
			this.ResultTextBox.Text = "0";
		}

		#endregion

		#region +ボタンクリック

		/// <summary>
		/// 入力されている値とこれから入力される値の足し算を実施するように設定します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void AddButton_Click(object sender, EventArgs e)
		{
			this.UpdateResult();
			this.operation = Operator.Add;
		}

		#endregion

		#region -ボタンクリック

		/// <summary>
		/// 入力されている値とこれから入力される値の引き算を実施するように設定します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SubButton_Click(object sender, EventArgs e)
		{
			this.UpdateResult();
			this.operation = Operator.Sub;
		}

		#endregion

		#region *ボタンクリック

		/// <summary>
		/// 入力されている値とこれから入力される値の掛け算を実施するように設定します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MultiButton_Click(object sender, EventArgs e)
		{
			this.UpdateResult();
			this.operation = Operator.Multi;
		}

		#endregion

		#region /ボタンクリック

		/// <summary>
		/// 入力されている値とこれから入力される値の割り算を実施するように設定します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void DevButton_Click(object sender, EventArgs e)
		{
			this.UpdateResult();
			this.operation = Operator.Dev;
		}

		#endregion

		#region =ボタンクリック

		/// <summary>
		/// 計算結果を表示します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void CalcButton_Click(object sender, EventArgs e)
		{
			this.UpdateResult();
			this.result = double.MinValue;
			this.operation = Operator.None;
		}

		#endregion

		#region Cボタンクリック

		/// <summary>
		/// 入力されている内容をクリアして結果フィールドに0を表示します
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ClearButton_Click(object sender, EventArgs e)
		{
			this.result = double.MinValue;
			this.isNew = true;
			this.operation = Operator.None;
			this.ResultTextBox.Text = "0";
		}

		#endregion

		#region 計算結果の更新

		/// <summary>
		/// 直前の値、入力された値、計算の種類から新しい値を計算します
		/// </summary>
		private void UpdateResult()
		{
			//throw new NotImplementedException();

			//値が入力されていたら処理を継続する
			if (!string.IsNullOrEmpty(this.ResultTextBox.Text))
			{
				//すでに値が保存されているか
				//保存されている場合は、(4)の時の計算処理
				if (this.result > double.MinValue)
				{
					//値がある場合は計算処理を行い、結果を反映
					this.Calculate();
					this.ResultTextBox.Text = this.result.ToString();
				}
				//保存されていない場合は、(2)の時の値保存処理
				else
				{
					//値が保存されていない場合は(1)の値を保存
					this.result = double.Parse(this.ResultTextBox.Text);
				}

				//処理をしたら新しい数字の入力を開始する
				this.isNew = true;
			}
		}

		#endregion

		#region 計算

		/// <summary>
		/// 現在の値に基づいて実際の計算を行い、結果をresult変数に格納します
		/// </summary>
		private void Calculate()
		{
			double current = double.Parse(this.ResultTextBox.Text);

			switch (this.operation)
			{
				case Operator.Add:
					this.result = this.result + current;
					break;
				case Operator.Sub:
					this.result = this.result - current;
					break;
				case Operator.Multi:
					this.result = this.result * current;
					break;
				case Operator.Dev:
					this.result = this.result / current;
					break;
			}
		}

		#endregion

		#region 0ボタンクリック

		private void ZeroButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("0");
		}

		#endregion

		#region 1ボタンクリック

		private void OneButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("1");
		}

		#endregion

		#region 2ボタンクリック

		private void TwoButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("2");
		}

		#endregion

		#region 3ボタンクリック

		private void ThreeButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("3");
		}

		#endregion

		#region 4ボタンクリック

		private void FourButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("4");
		}

		#endregion

		#region 5ボタンクリック

		private void FiveButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("5");
		}

		#endregion

		#region 6ボタンクリック

		private void SixButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("6");
		}

		#endregion

		#region 7ボタンクリック

		private void SevenButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("7");
		}

		#endregion

		#region 8ボタンクリック

		private void EightButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("8");
		}

		#endregion

		#region 9ボタンクリック

		private void NineButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay("9");
		}

		#endregion

		#region .ボタンクリック

		private void DotButton_Click(object sender, EventArgs e)
		{
			this.UpdateDisplay(".");
		}

		#endregion

		#region 画面表示更新

		/// <summary>
		/// 押された数字ボタンの種類に応じて画面の表示内容を更新します
		/// </summary>
		/// <param name="number"></param>
		private void UpdateDisplay(string number)
		{
			//新しく数字の入力を開始する場合
			if (this.isNew == true)
			{
				//初めに . が押された場合は"0."と表示する
				if (number == ".")
				{
					this.ResultTextBox.Text = "0.";
					this.isNew = false;
				}
				//1文字目が0の場合は表示だけを更新
				else if (number == "0")
				{
					this.ResultTextBox.Text = number;
				}
				//新しい数字の入力を開始するときは0以外のときにする
				else
				{
					this.ResultTextBox.Text = number;
					this.isNew = false;
				}
			}
			//既存の数字に追加する場合
			else
			{
				this.ResultTextBox.Text += number;
			}
		}

		#endregion

	}
}
