﻿using System;
using System.Text;
using System.Windows.Forms;

namespace MyWallet
{
  public partial class FormMain : Form
  {
    const double VERSION = 0.02; //2009/01/23
    const string APPLICATION_NAME = "MyWallet";
    const int COLUMN_COUNT = 6;
    const string PAID_STRING = "済";
    const string NO_PAID_STRING = "未";

    string InternalFileName = "";
    string FileName
    {
      get
      {
        return InternalFileName;
      }
      set
      {
        this.InternalFileName = value;
        if (value != "")
          this.Text = APPLICATION_NAME + " - " + value;
        else
          this.Text = APPLICATION_NAME;
      }
    }

    public FormMain()
    {
      InitializeComponent();
    }

    private void FormMain_Load(object sender, EventArgs e)
    {
      openFileDialog1.FileName = "";
      openFileDialog1.Filter = 
        "テキスト・ファイル(*.txt)|*.txt|すべてのファイル(*.*)|*.*";
      saveFileDialog1.Filter = "テキスト・ファイル(*.txt)|*.txt";

      dataGridViewMain.Dock = DockStyle.Fill;
      string[] ColumnName = new string[COLUMN_COUNT] 
        { "ID", "年月日", "店舗", "金額", "備考", "支払済？" };
      dataGridViewMain.ColumnCount = COLUMN_COUNT;
      for (int i = 0; i < COLUMN_COUNT; i++)
        dataGridViewMain.Columns[i].Name = ColumnName[i];
      dataGridViewMain.ColumnHeadersVisible = true;
      dataGridViewMain.Columns[5].ReadOnly = true;
      
      InitializeData();
    }

    void InitializeData()
    {
      FileName = "";
      dataGridViewMain.Rows.Clear();
    }

    private void ToolStripMenuItemFileExit_Click(object sender, 
      EventArgs e)
    {
      Application.Exit();
    }

    private void ToolStripMenuItemFileVersion_Click(object sender, 
      EventArgs e)
    {
      MessageBox.Show(APPLICATION_NAME + Environment.NewLine +
        "Version " + VERSION.ToString() + Environment.NewLine +
        "(c)2009 Nikkei Software", "バージョン情報");
    }

    private void ToolStripMenuItemFileNew_Click(object sender, 
      EventArgs e)
    {
      InitializeData();
    }

    private void ToolStripMenuItemFileOpen_Click(object sender, 
      EventArgs e)
    {
      DialogResult Result = openFileDialog1.ShowDialog();
      if (Result == DialogResult.OK)
      {
        FileName = openFileDialog1.FileName;
        string DataText = System.IO.File.ReadAllText(FileName, 
          Encoding.GetEncoding("Shift_JIS"));
        int LineCounter = DataText.Length - 
          DataText.Replace("\r\n", "\r").Length;
        if (LineCounter % COLUMN_COUNT != 0)
          MessageBox.Show("データの行数が" + COLUMN_COUNT.ToString() +
            "で割り切れません");
        else
        {
          string[] Delimiter = {"\r\n"};
          string[] StringArray = DataText.Split(Delimiter, 
            StringSplitOptions.None);
          for (int i = 0; i < LineCounter; i = i + COLUMN_COUNT)
          {
            string[] row = new string[COLUMN_COUNT];
            for (int j = 0; j < COLUMN_COUNT; j++)
              row[j] = StringArray[i + j];
            dataGridViewMain.Rows.Add(row);
          }
        }
      }
    }

    private void ToolStripMenuItemFileSave_Click(object sender,
      EventArgs e)
    {
      string s = "";
      for (int i = 0; i < dataGridViewMain.RowCount - 1; i++)
        for (int j = 0; j < COLUMN_COUNT; j++)
          try
          {
            s = s + dataGridViewMain.Rows[i].Cells[j].Value.ToString() + "\r\n";
          }
          catch
          {
            s = s + "\r\n";
          }
          if (s == "")
        MessageBox.Show("保存すべきデータがありません");
      else
        System.IO.File.WriteAllText(FileName, s, 
          Encoding.GetEncoding("Shift_JIS"));
    }

    private void ToolStripMenuItemSaveAs_Click(object sender, EventArgs e)
    {
      DialogResult Result = saveFileDialog1.ShowDialog();
      if (Result == DialogResult.OK)
      {
        FileName = saveFileDialog1.FileName;
        ToolStripMenuItemFileSave_Click(sender, e);
      }
    }

    private void ToolStripMenuItemFile_Click(object sender, EventArgs e)
    {
      if (FileName == "")
        ToolStripMenuItemFileSave.Enabled = false;
      else
        ToolStripMenuItemFileSave.Enabled = true;
    }

    private void dataGridViewMain_CellDoubleClick(object sender, 
      DataGridViewCellEventArgs e)
    {
      if (e.ColumnIndex == 5)
      {
        string s;
        try
        {
          s = dataGridViewMain.Rows[e.RowIndex].Cells[e.ColumnIndex].
            Value.ToString();
        }
        catch
        {
          s = PAID_STRING;
        }

        if (s == PAID_STRING)
        dataGridViewMain.Rows[e.RowIndex].Cells[e.ColumnIndex].Value = 
          NO_PAID_STRING;
        else
          dataGridViewMain.Rows[e.RowIndex].Cells[e.ColumnIndex].Value = 
            PAID_STRING;
        dataGridViewMain.Update();
      }
    }

    private void dataGridViewMain_KeyUp(object sender, KeyEventArgs e)
    {
      try
      {
        if (e.KeyValue == 46)
          if (MessageBox.Show("行を削除しますか？", "行削除の確認",
            MessageBoxButtons.YesNo) == DialogResult.Yes)
            dataGridViewMain.Rows.Remove(dataGridViewMain.CurrentRow);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "エラー");
      }
    }

    private void ToolStripMenuItemCalculateAddUp_Click(object sender, 
      EventArgs e)
    {
      int Total = 0;
      for (int i = 0; i < dataGridViewMain.RowCount - 1; i++)
        if (dataGridViewMain.Rows[i].Cells[5].Value.ToString() ==
          NO_PAID_STRING)
          Total = Total +
            int.Parse(dataGridViewMain.Rows[i].Cells[3].Value.ToString());
      MessageBox.Show("次回の請求は" + Total.ToString() + "円です");
    } 
  }
}
